@@ -16,7 +16,7 @@ from django.core.serializers import python
 from django.core.exceptions import ImproperlyConfigured
 from django.db import connections
 from django.db.models import query
-from django.db.models.sql import Query, constants, subqueries
+from django.db.models.sql import Query, RawQuery, constants, subqueries
 from django.db.backends.signals import connection_created
 from django.utils.encoding import force_unicode
 
@@ -164,6 +164,15 @@ def get_resource(url):
 	log.debug('Request API URL: %s' % url)
 	return resource
 
+class SalesforceRawQuerySet(query.RawQuerySet):
+	def __len__(self):
+		if(self.query.cursor is None):
+			# force the query
+			self.query.get_columns()
+			return len(self.query.cursor.results)
+		else:
+			return 0;
+
 class SalesforceQuerySet(query.QuerySet):
 	"""
 	Use a custom SQL compiler to generate SOQL-compliant queries.
@@ -181,6 +190,25 @@ class SalesforceQuerySet(query.QuerySet):
 		for res in python.Deserializer(pfd(self.model, r, self.db) for r in cursor.results):
 			yield res.object
 
+class SalesforceRawQuery(RawQuery):
+	def clone(self, using):
+		return SalesforceRawQuery(self.sql, using, params=self.params)
+
+	def get_columns(self):
+		if self.cursor is None:
+			self._execute_query()
+		converter = connections[self.using].introspection.table_name_converter
+		if(len(self.cursor.results) > 0):
+			return [converter(col) for col in self.cursor.results[0].keys() if col != 'attributes']
+		return []
+
+	def _execute_query(self):
+		self.cursor = CursorWrapper(connections[self.using], self)
+		self.cursor.execute(self.sql, self.params)
+	
+	def __repr__(self):
+		return "<SalesforceRawQuery: %r>" % (self.sql % tuple(self.params))
+
 class SalesforceQuery(Query):
 	"""
 	Override aggregates.
@@ -188,6 +216,9 @@ class SalesforceQuery(Query):
 	from salesforce.backend import aggregates
 	aggregates_module = aggregates
 	
+	def clone(self, klass=None, memo=None, **kwargs):
+		return Query.clone(self, klass, memo, **kwargs)
+	
 	def has_results(self, using):
 		q = self.clone()
 		compiler = q.get_compiler(using=using)
@@ -207,7 +238,7 @@ class CursorWrapper(object):
 		connection_created.send(sender=self.__class__, connection=self)
 		self.settings_dict = conn.settings_dict
 		self.query = query
-		self.results = iter([])
+		self.results = []
 		self.rowcount = None
 	
 	@property
@@ -222,6 +253,8 @@ class CursorWrapper(object):
 		
 		if(isinstance(self.query, SalesforceQuery)):
 			response = self.execute_select(q, args)
+		elif(isinstance(self.query, SalesforceRawQuery)):
+			response = self.execute_select(q, args)
 		elif(isinstance(self.query, subqueries.InsertQuery)):
 			response = self.execute_insert(self.query)
 		elif(isinstance(self.query, subqueries.UpdateQuery)):
@@ -253,7 +286,7 @@ class CursorWrapper(object):
 			
 			self.results = self.query_results(data)
 		else:
-			self.results = iter([])
+			self.results = []
 	
 	def execute_select(self, q, args):
 		processed_sql = q % process_args(args)
@@ -312,9 +345,10 @@ class CursorWrapper(object):
 		return handle_api_exceptions(url, resource.delete, headers=headers)
 	
 	def query_results(self, results):
+		output = []
 		while True:
 			for rec in results['records']:
-				yield rec
+				output.append(rec)
 
 			if results['done']:
 				break
@@ -327,13 +361,17 @@ class CursorWrapper(object):
 				results = json.loads(jsrc)
 			else:
 				break
+		return output
+	
+	def __iter__(self):
+		return iter(self.results)
 
 	def fetchone(self):
 		"""
 		Fetch a single result from a previously executed query.
 		"""
 		try:
-			return next(self.results)
+			return self.results.pop(0)
 		except StopIteration:
 			return None
 
